\begin{code}
{-# OPTIONS -fglasgow-exts #-}
module Box where
import Monoid
import Test.QuickCheck
import List(sort)
\end{code}

Boxes.  A _box_ encloses a walk on the line: it has an input, output,
maximum, and minimum.  We are particularly interested in whether or
not the maximum and minimum are unique, since that determines whether
or not the group fibers.  We are only interested in boxes up to
translation (when composing, we translate so the input of one matches
the output of the other), so we normalize so that it starts at 0.

We represent a box with 5 numbers:
* The final position of $w$, called _shift_.
* The maximum value of $w$, called _top_
* The minimum value of $w$, called _bot_.
* The number of times the top is visited, counting visits at the
  first and last spot once and all others twice, called _ntop_.  We
  count in this way to make boxes behave well under concatenation and
  reversal.
* The number of times the bottom is visited, counted in the same way,
  denoted _nbot_.

\begin{code}
data Reps = Reps { val, reps :: !Integer }
   deriving (Eq, Show, Read)

data Box = Box { shift :: !Integer, top, bot :: !Reps }
   deriving (Eq, Show, Read)
\end{code}

\begin{code}
unitBox = Box 0 (Reps 0 0) (Reps 0 0)

-- The elementary building blocks of boxes
primBox :: Integer -> Box
primBox d | d > 0 =  Box d (Reps d 1) (Reps 0 1)
          | d < 0 =  Box d (Reps 0 1) (Reps d 1)

-- Special cases when one of the generators is flat
primFlatBox :: Box
primFlatBox = Box 0 (Reps 0 2) (Reps 0 2)

primShiftBox :: Integer -> Box
primShiftBox d | d >= 0    = Box d (Reps 0 0) (Reps d 0)
	       | otherwise = Box d (Reps d 0) (Reps 0 0)

translateReps :: Integer -> Reps -> Reps
translateReps s (Reps v n) = Reps (v+s) n

translateBox :: Integer -> Box -> Box
translateBox s b =
  Box (s + shift b) (translateReps s (top b)) (translateReps s (bot b))

instance Monoid Box where
    unit = unitBox
    b1 & b2 =
      let b2' = translateBox (shift b1) b2
          top' = combineReps compare (top b1) (top b2')
          bot' = combineReps (flip compare) (bot b1) (bot b2')
      in Box (shift b2') top' bot'

combineReps :: (Integer -> Integer -> Ordering) -> Reps -> Reps -> Reps
combineReps comp r1 r2 =
  case comp (val r1) (val r2) of
    GT -> r1
    LT -> r2
    EQ -> Reps (val r1) (reps r1 + reps r2)

instance RevMonoid Box where
    rev b = (translateBox (-shift b) b) { shift = -shift b }
\end{code}

A box is _marked_ if ntop or nbot are too large.

\begin{code}
markedReps :: Reps -> Bool
markedReps (Reps _ r) = r > 2

markedTop, markedBot, marked :: Box -> Bool
markedTop = markedReps . top
markedBot = markedReps . bot
marked b = markedTop b || markedBot b
\end{code}

Marks are persistent.
\begin{code}
prop_markedTop_persist, prop_markedBot_persist :: Box -> Box -> Property
prop_markedTop_persist b1 b2 = (markedTop b1 && markedTop b2) ==>
				  markedTop (b1 & b2)

prop_markedBot_persist b1 b2 = (markedBot b1 && markedBot b2) ==>
				  markedBot (b1 & b2)
\end{code}

In order to test this code, we need to generate random boxes.

This uses QuickCheck.  For more information, see
http://www.math.chalmers.se/~rjmh/QuickCheck/ .

\begin{code}
instance Arbitrary Reps where
  arbitrary = sized $ \sz ->
    do val <- arbitrary
       reps <- choose (0, fromIntegral sz)
       return (Reps val reps)

  coarbitrary (Reps v r) = coarbitrary v . coarbitrary r

instance Arbitrary Box where
  arbitrary = sized $ \sz0 -> 
    do let sz = fromIntegral sz0
       shift <- arbitrary
       (top, bot) <- arbitrary
       return (Box shift (translateReps (sz + max shift 0) top)
	                 (translateReps (-sz + min shift 0) bot))
  coarbitrary (Box shift top bot) =
    coarbitrary shift . coarbitrary top . coarbitrary bot

validBox :: Box -> Bool
validBox b = val (top b) >= max 0 (shift b) && min 0 (shift b) >= val (bot b)

prop_valid_valid b1 b2 = validBox (b1 & b2)
\end{code}

A convenience function for testing: specialize the generic
properties defined in Monoid.lhs for 'Box'.

\begin{code}
testBox :: (Testable (Box -> a)) => (Box -> a) -> IO ()
testBox = test
\end{code}

Copyright 2002, 2004, 2005 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
