\begin{code}
module FreeGroup where
import Monoid
import Char(ord, chr, readLitChar)
import Maybe(maybeToList)
\end{code}

Free groups.  Note that there is no cancellation implemented here.
Fortunately, all the words we generate end up being reduced in the
free group.

\begin{code}
data FreeGroupGen = FGPos {gen:: !Int} | FGNeg {gen:: !Int} deriving Eq

fgsign :: (Num a) => FreeGroupGen -> a
fgsign (FGPos _) = 1
fgsign (FGNeg _) = -1

data FreeGroup = FG [FreeGroupGen] deriving Eq

instance Monoid FreeGroup where
  unit = FG []
  (FG w1) & (FG w2) = FG (w1 ++ w2)

instance RevMonoid FreeGroup where
  rev (FG l) = FG (reverse (map neg l)) where
    neg (FGPos x) = FGNeg x
    neg (FGNeg x) = FGPos x
\end{code}

To print out free group elements properly.

\begin{code}
fgToChar :: FreeGroupGen -> Char
fgToChar (FGPos n) | n >= 0 && n <= 26 = chr (ord 'a' + n)
fgToChar (FGNeg n) | n >= 0 && n <= 26 = chr (ord 'A' + n)
fgToChar _ | otherwise = error "fgToChar: generator out of range"

charToFG :: Char -> Maybe FreeGroupGen
charToFG c | ord 'a' <= ord c && ord c <= ord 'z'
    = Just (FGPos (ord c - ord 'a'))
charToFG c | ord 'A' <= ord c && ord c <= ord 'Z'
    = Just (FGNeg (ord c - ord 'A'))
charToFG c | otherwise
    = Nothing

instance Show FreeGroupGen where
  show x = [fgToChar x]

instance Read FreeGroupGen where
  readsPrec p s = [(x, t) | (c, t) <- readLitChar s,
			    x <- maybeToList (charToFG c)]

instance Show FreeGroup where
  showsPrec _ (FG l) = showChar '<' . foldr (.) id (map shows l) . showChar '>'

instance Read FreeGroup where
  readsPrec p s = [(FG l, u) | ("<", t) <- lex s,
			       (l, u) <- readl t]
      where readl ('>':s) = [([], s)]
	    readl (c:s)   = [(g:gs,t) | g <- maybeToList (charToFG c),
				        (gs,t) <- readl s]
	    readl ""      = []

fgid, fga, fgA, fgb, fgB, fgc, fgC, fgd, fgD :: FreeGroup
fgid = FG []
[fga, fgA, fgb, fgB, fgc, fgC, fgd, fgD] =
  map (\c -> FG [read c])
      ["a", "A", "b", "B", "c", "C", "d", "D"]
\end{code}

Copyright 2002, 2004, 2005 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
