\begin{code}
module Genus2 where
import DT
import PIL
import qualified List
\end{code}

A genus 2 surface has a pair of pants decomposition with 3 curves.
There are two different types, depending on whether the graph obtained
by collapsing the pair of pants to a trivalent vertex looks like a
theta graph or a barbell graph.

\begin{code}
data Genus2 = Barbell { e1,e2,e3 :: DTAnnulus } |
	      Theta   { e1,e2,e3 :: DTAnnulus }
    deriving (Show, Eq, Read)

curveLength :: Genus2 -> Integer
curveLength g = m (e1 g) + m (e3 g)
\end{code}

A curve is represented with respect to a given marking of the surface.
Sometimes when we do a move, there is no "natural" marking of the new
surface; rather, the new marking lies in between two possible
markings, and we need to pick one.  To avoid any bias in the random
generation, we keep track of the difference between the current
marking and the most natural one, in units of eigths of a full twist.

\begin{code}
type Genus2T = (Genus2, (Integer, Integer, Integer))
\end{code}

Operations to perform symmetries on the underlying graph.

\begin{code}
reflect :: Genus2 -> Genus2
reflect g2 = g2 { e1 = e3 g2, e3 = e1 g2 }
reflectT :: Genus2T -> Genus2T
reflectT (g2,(n1,n2,n3)) = (reflect g2,(n3,n2,n1))

rotate :: Genus2 -> Genus2
rotate g2@(Theta {}) = g2 { e1 = e3 g2, e2 = e1 g2, e3 = e2 g2 }
rotateT :: Genus2T -> Genus2T
rotateT (g2, (n1,n2,n3)) = (rotate g2, (n3,n1,n2))
\end{code}

Check that all the parity conditions are satisfied.  Mainly for debugging.

\begin{code}
verifyParity :: Genus2 -> Bool
verifyParity (Barbell e1 e2 e3) =
   even (m e2) && even (t e1) && even (t e3) &&
     even ((t e2) + lk (m e2) (m e1) (m e1) + lk (m e2) (m e3) (m e3))
verifyParity (Theta e1 e2 e3) =
	even ((m e1) + (m e2) + (m e3))
      && even (t e1) && even (t e2) && even (t e3)
\end{code}

Copyright 2002, 2004 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
