\begin{code}
{-# OPTIONS -fglasgow-exts #-}
module Interval where
import PIL((//))
import Monoid(Monoid(unit, (&)), RevMonoid(rev))
\end{code}

Intervals
---------
An _interval_ represents the subset [a, b) of the integers.

\begin{code}
type Index = Integer

data Interval = Interval {left, right :: !Index}
		  deriving (Eq, Show, Ord, Read)

l ... r = Interval {left = l, right = r}

class HasWidth a where
    width :: a -> Index

instance HasWidth Interval where
    width i = right i - left i

inRange :: Interval -> Index -> Bool
inRange (Interval l r) x = x >= l && x < r
\end{code}

Pairings
--------
A _pairing_ is an isometry between two intervals.

\begin{code}
data Orient = Preserve | Reverse deriving (Eq, Show, Read)
data Pairing a = Pairing {domain, range :: !Interval,
			  orient :: !Orient,
			  dat :: a} deriving (Eq, Show, Read)

instance Functor Pairing where
    fmap f p = p {dat = f (dat p)}
	       
i0 ..> i1 = Pairing {domain = i0, range = i1,
		     orient = Preserve,
		     dat = undefined}

i0 .*> i1 = Pairing {domain = i0, range = i1,
		     orient = Reverse,
		     dat = undefined}

instance HasWidth (Pairing a) where
    width p = width (domain p)

setData :: b -> Pairing a  -> Pairing b
setData b p = fmap (const b) p

killData :: Pairing a -> Pairing ()
killData = setData ()
\end{code}

Pairings form a partially defined monoid.

\begin{code}
instance Monoid Orient where
    unit = Preserve
    Preserve & Preserve = Preserve
    Preserve & Reverse  = Reverse
    Reverse  & Preserve = Reverse
    Reverse  & Reverse  = Preserve

instance Monoid a => Monoid (Pairing a) where
    unit = error "Pairing monoid: No unit"
    p & q | range p == domain q = Pairing {domain = domain p,
					   range = range q,
					   orient = orient p & orient q,
					   dat = dat p & dat q}
	  | otherwise        = error "Pairing monoid: Illegal composition"

instance RevMonoid a => RevMonoid (Pairing a) where
    rev p = Pairing {domain = range p,
		     range  = domain p,
		     orient = orient p,
		     dat    = rev (dat p)}
\end{code}

It is sometimes convenient to normalize pairings so that they move to
the right.

\begin{code}
moveRight :: RevMonoid a => Pairing a -> Pairing a
moveRight p | domain p <= range p = p
            | otherwise           = rev p
\end{code}

If the domain and range of a pairing are equal, we may count the
number of orbits created.

\begin{code}
reduceIdentity :: Monoid a => Pairing a -> [(Index, a)]
reduceIdentity p | domain p /= range p =
		     error "reduceIdentity: domain != range"
		 | orient p == Preserve = [(width p, dat p)]
		    -- Orientation reversing. Untested, semantics not clear?
		 | even (width p) = [(width p `div` 2, dat p & dat p)]
		 | odd  (width p) = [(1, dat p),
				     (width p `div` 2, dat p & dat p)]
\end{code}

A _periodic_ pairing (Agol, Hass, Thurston) is an
orientation-preserving pairing for which the domain and range overlap.
We need to take special care of these, because it might take a very
long time to shuttle other pairings to the lower end of the domain.

\begin{code}
periodic :: (RevMonoid a) => Pairing a -> Bool
periodic p | orient p == Reverse = False
periodic p = let p' = moveRight p in right (domain p') > left (range p')
\end{code}

cutPairing: cut a pairing at a point in the domain.

\begin{code}
cutPairing :: Pairing a -> Index -> (Pairing a, Pairing a)
cutPairing p x | not (inRange (domain p) x) =
		   error "cutPairing: point not in domain"
	       | orient p == Preserve =
		   let Interval x0 x1 = domain p
		       Interval y0 y1 = range p
		       y = y0 + x - x0
		   in (((x0...x) ..> (y0...y)){dat = dat p},
		       ((x...x1) ..> (y...y1)){dat = dat p})
	       | orient p == Reverse =
		   let Interval x0 x1 = domain p
		       Interval y0 y1 = range p
		       y = y1 - x + x0
		   in (((x0...x) .*> (y...y1)){dat = dat p},
		       ((x...x1) .*> (y0...y)){dat = dat p})
\end{code}

Constructing pairings
---------------------

To easily construct intervals, we have makeIntervals which is
given a list of lengths and constructs a list of non-overlapping
intervals of the given lengths.

\begin{code}
makeIntervals          :: Interval -> [Integer] -> [Interval]
makeIntervals i [] | left i == right i = []
makeIntervals i (l:ls) = (s...(s+l)):makeIntervals i{left=s+l} ls
    where s = left i
makeIntervals _ _ = error "makeIntervals: not a subdivision"
\end{code}

One way to construct pairings is at a trivalent vertex: given any
three intervals (with total width even), there is a unique way to
connect them without trivial loops (modulo rotations of the boundary).

\begin{code}
triConnect, triConnectPlain, triConnectWide
    :: Interval -> Interval -> Interval -> [Pairing ()]
triConnect i0 i1 i2 =
  let {w0 = width i0; w1 = width i1; w2 = width i2} in
  if (w0 > w1 + w2) then triConnectWide i0 i1 i2
  else if (w1 > w0 + w2) then triConnectWide i1 i2 i0
  else if (w2 > w0 + w1) then triConnectWide i2 i0 i1
  else triConnectPlain i0 i1 i2

triConnectPlain i0 i1 i2 =
  let w0 = width i0; w1 = width i1; w2 = width i2
      c01 = (w0 + w1 - w2) // 2
      c12 = (w1 + w2 - w0) // 2
      c20 = (w2 + w0 - w1) // 2
      [i01, i02] = makeIntervals i0 [c01, c20]
      [i12, i10] = makeIntervals i1 [c12, c01]
      [i20, i21] = makeIntervals i2 [c20, c12]
  in sanitize ([i01 .*> i10, i12 .*> i21, i20 .*> i02])

triConnectWide i0 i1 i2 =
  let w1 = width i1
      w2 = width i2
      c00 = (width i0 - w1 - w2) // 2
      [i01, i00, i02, i00'] = makeIntervals i0 [w1, c00, w2, c00]
  in sanitize ([i01 .*> i1, i02 .*> i2, i00 .*> i00'])
\end{code}

For another type of connection, we may take two intervals of the same
width and join them with a cyclic shift.

\begin{code}
cyclicConnect :: Interval -> Interval -> Integer -> [Pairing ()]
cyclicConnect i0 i1 shift0 =
    let w = width i0
	shift = shift0 `mod` w
	[i0', i0''] = makeIntervals i0 [shift, w-shift]
	[i1', i1''] = makeIntervals i1 [shift, w-shift]
    in if w <= 0 then [] else sanitize ([i0' .*> i1', i0'' .*> i1''])

plainConnect :: Interval -> Interval -> [Pairing ()]
plainConnect i0 i1 = sanitize [i0 .*> i1]
\end{code}

To make things neater, we filter the output so that 0-width pairings
do not occur.

\begin{code}
sanitize :: [Pairing a] -> [Pairing ()]
sanitize ps = map killData (filter (\p -> width p > 0) ps)
\end{code}

Copyright 2002, 2004 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
