\begin{code}
module IntervalExchange where

import Monoid(Monoid(unit, (&), pow), RevMonoid(rev))
import Interval
import List(partition, findIndex)
\end{code}

Here we implement _interval_exchanges_: piecewise isometric maps of an
interval to itself.  (Actually, we allow a more general structure, in
which one side of a strip may be attached to itself.  Think of a
thickened interval with bands attached.)

Such a structure may be thought of as a groupoid of pairings, as in
the Agol-Hass-Thurston paper, with the additional restriction that
exactly two pairings involve any given point on the surface.

\begin{code}
type IntervalExchange a = [Pairing a]

extractMax :: IntervalExchange a -> ([Pairing a], [Pairing a])
extractMax ps = let m = maximum (map (right . range) ps) in
    partition (\p -> right (range p) >= m) ps

contractPeriodic :: (RevMonoid a) => Pairing a -> [Pairing a]
contractPeriodic p' =
    let p = moveRight p'
	w = width p
	(x0, x1, x2, x3) = (left (domain p), left (range p),
			    right (domain p), right (range p))
	t = x1 - x0
	(n, m) = w `divMod` t
	a = dat p
	an = a `pow` n
	an' = an & a
    in case m of 0 -> [((x0...x1) ..> (x2...x3)){dat = an}]
		 _ -> [((x0...(x0+m)) ..> ((x3-m)...x3)){dat = an'},
		       (((x0+m)...x1) ..> (x2...(x3-m))){dat = an}]

stepIE :: RevMonoid a => IntervalExchange a -> (IntervalExchange a, [(Integer, a)])
stepIE ps' =
    let ps = map moveRight ps'
	(mxs, rest) = extractMax ps
	doCut p1 p2 | width p1 == width p2 = [p1 & rev p2]
		    | width p1 > width p2  = doCut p2 p1
		    | periodic p2 = p1:contractPeriodic p2
		    | otherwise =
		   let p2' = rev p2
		       (p2'l, p2'r) = cutPairing p2' (left (range p1))
		   in [p2'l, p1 & p2'r]
    in case mxs of
        [p]      -> (rest, reduceIdentity p)
	[p1, p2] -> (doCut p1 p2 ++ rest, [])
	_        -> error "stepIE: strangeness"

emptyIE :: IntervalExchange a -> Bool
emptyIE = null

runIE1 :: RevMonoid a => IntervalExchange a -> [(Integer, a)]
runIE1 ie | emptyIE ie = []
	  | otherwise = let (ie', l) = stepIE ie in l++runIE1 ie'
\end{code}

Combine repeated occurrences of the same element in the result of an
interval exchange.

\begin{code}
combineRepeats :: (Eq a) => [(Integer, a)] -> [(Integer, a)]
combineRepeats [] = []
combineRepeats ((i0,x0):xs) =
    let (sames, others) = partition (\x -> snd x == x0) xs
    in (i0 + sum (map fst sames), x0):combineRepeats others

runIE :: (Eq a, RevMonoid a) => IntervalExchange a -> [(Integer, a)]
runIE ie = combineRepeats (runIE1 ie)
\end{code}

Sometimes we want to stop running the interval exchange early.

\begin{code}
runIEstop :: (Eq a, RevMonoid a) => (IntervalExchange a -> Bool) ->
    IntervalExchange a -> (Int, Maybe [(Integer, a)])
runIEstop stop ie = runIEstop' 0 [] ie
  where runIEstop' n r ie =
	  if stop ie then (n, Nothing)
	  else if emptyIE ie then (n, Just (combineRepeats r))
	  else let (ie', s) = stepIE ie in
	       runIEstop' (n+1) (s++r) ie'
\end{code}

Copyright 2002, 2004 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
