\begin{code}
module Main where
import RandomGenus2
import RandomMoves
import Genus2
import Genus2Moves
import Genus2MCG
import Genus2IE
import Genus2Simplify
import Util(relax, readListLn, showListLn, log2)
import Statistics

import System.Console.GetOpt
import System.Environment
import System.Exit
import IO
import Monad
import Random
import Maybe(catMaybes)
\end{code}

The possible sources for the lamination to check.

\begin{code}
data SourceOpt =
    SourceML |
    SourceMCG |
    SourcePants |
    SourceFile (IO String)
\end{code}

The different options that control the program.

\begin{code}
data Options = Options {
		optSeed :: Maybe Int,
		optSource :: SourceOpt,
		optSize :: Int,
                optWalkRepetitions :: Int,
		optVariance :: Double,
		optNumTrials :: Int,
		optOutput :: String -> IO (),
		optIntermediate :: String -> IO (),
		optExtraTwists :: Maybe Int }

defaultOptions :: Options
defaultOptions = Options {
		optSeed = Nothing,
		optSource = SourceMCG,
		optSize = 100,
                optWalkRepetitions = 1,
		optVariance = 1.0,
		optNumTrials = 20,
		optOutput = putStrLn,
		optIntermediate = const relax,
		optExtraTwists = Nothing }

options :: [ OptDescr (Options -> IO Options) ]
options =
  [ Option "l" ["lamination"]
      (NoArg (\opt -> do return opt { optSource = SourceML }))
      "Curves from random measured lamination",
    Option "m" ["mcg"]
      (NoArg (\opt -> do return opt { optSource = SourceMCG }))
      "Curves from a random walk in mapping class group",
    Option "p" ["pants"]
      (NoArg (\opt -> do return opt { optSource = SourcePants }))
      "Curves from a walk in the pants complex",
    Option "f" ["file"]
      (ReqArg (\arg opt ->
	         let fileSource = case arg of "-" -> getContents
					      _   -> readFile arg
		 in do return opt { optSource = SourceFile fileSource })
	      "FILE")
	"Read test case from file ('-' for stdin)",
    Option "s" ["size"]
      (ReqArg (\arg opt ->
        do { a <- readArg arg ; return opt { optSize = a } })
	      "SIZE")
      "Size of test cases to generate",
    Option "n" ["numtrials"]
      (ReqArg (\arg opt ->
        do { a <- readArg arg ; return opt { optNumTrials = a } })
	      "NUM")
      "Number of trials to run",
    Option "v" ["variance"]
      (ReqArg (\arg opt ->
        do { a <- readArg arg ; return opt { optVariance = a } })
	      "STDDEV")
      "Standard deviation of twists (pants complex only)",
    Option "i" ["intermediate"]
      (ReqArg (\arg opt ->
		 do f <- openFile arg WriteMode
		    return opt { optIntermediate = \s ->
				 do {hPutStrLn f s; hFlush f}})
	      "FILE")
      "Write test cases to file",
    Option "S" ["seed"]
      (ReqArg (\arg opt ->
        do { a <- readArg arg ; return opt { optSeed = Just a } })
	      "SEED")
      "Random number seed",
    Option "h" ["help"]
      (NoArg (\_ -> do { showHelp; exitWith ExitSuccess }))
      "Print this help message",
    Option "e" ["extra-twists"]
      (OptArg (\ar opt ->
         case ar of
	   Nothing -> return opt {optExtraTwists = Just 1}
	   Just arg -> do { a <- readArg arg ; return opt { optExtraTwists = Just a } })
      "TWISTS")
      "Add some extra twists at the end",
    Option "r" ["repeat"]
      (ReqArg (\arg opt -> do {a <- readArg arg; return opt { optWalkRepetitions = a } })
              "NUM")
      "Number of times to repeat the walk (MCG and pants complex only)"
  ]

readArg :: (Read a) => String -> IO a
readArg arg =
   case reads arg of
     [(v,"")] -> return v
     _ -> do putStrLn ("Couldn't parse argument " ++ arg)
	     exitFailure
\end{code}
Write a usage message.

\begin{code}
showHelp :: IO ()
showHelp = do
  prg <- getProgName
  hPutStrLn stderr (usageInfo prg options)
  hFlush stderr

exitErrorHelp :: String -> IO a
exitErrorHelp msg = do
  hPutStrLn stderr msg
  hPutStrLn stderr ""
  showHelp
  exitFailure
\end{code}

The various kinds of input we accept.

\begin{code}
data TestCase = TestCurve Genus2 |
		TestMCG	  MCWord |
		TestPants [Move]
    deriving (Eq, Show, Read)

getCurve :: TestCase -> Genus2
getCurve (TestCurve c) = c
getCurve (TestMCG   w) = standardForm $ simplify $ doMCG w simpleCurve
getCurve (TestPants m) = standardForm $ simplify $ fst $ doMovesT m simpleCurveT

doTest :: Options -> Genus2 -> IO (Double, Result)
doTest opts c =
  do (optIntermediate opts) (show c)
     let ie = gen2ToIE c
	 size = log2 $ curveLength c
	 res = doIE ie
     putStrLn $ formatResult res
     return (size, res)

printSummary :: [(Double, Result)] -> IO ()
printSummary l =
  do let avgSize = average (map fst l)
	 ans = catMaybes $ map (fibers.snd) l
         numFiber = fi $ length $ filter (== True) ans
         numNoFiber = fi $ length $ filter (== False) ans
     putStrLn ("Average size: 2^"++show avgSize)
     putStrLn ((show $ numFiber*100 / (numFiber + numNoFiber)) ++ "% fiber")
\end{code}

Add some extra twists to test cases.

\begin{code}
addExtraTwists :: (RandomGen g) => Int -> g -> [Genus2] -> [Genus2]
addExtraTwists n g cs =
  let extras = randMoves 100 n g
      addTwists :: Genus2 -> [Move] -> Genus2
      addTwists g ms = fst $ last $ scanl (flip moveT) (g,(0,0,0)) ms
  in map (standardForm . simplify) $ zipWith addTwists cs extras
\end{code}

The size of curves to take.

\begin{code}
curveSize :: Int -> Integer
curveSize n = floor (1.2^n)
\end{code}

The main function.

\begin{code}
main = do
  args <- getArgs
  let (actions, rest, errors) = getOpt RequireOrder options args
  case rest of
    []     -> relax
    (r1:_) -> exitErrorHelp ("Unrecognized argument " ++ r1)
  when (not (null errors)) $ do
    mapM_ (hPutStrLn stderr) errors
    showHelp
    exitFailure
  opts <- foldl (>>=) (return defaultOptions) actions
  let s = optSize opts
      n = optNumTrials opts
      v = optVariance opts
      r = optWalkRepetitions opts
  seed <- case optSeed opts of
    Just s -> return s
    Nothing -> randomIO
  let g = mkStdGen seed
      g' = mkStdGen (seed+1)
  putStrLn ("Seed: " ++ show seed)
  tests <- case optSource opts of
    SourceML     -> return $ map TestCurve $ take n $ randCurves (curveSize s) g
    SourceMCG    -> return $ map TestMCG   $ take n $ map (concat.replicate r) $ randWords s g
    SourcePants  -> return $ map TestPants $ take n $ map (concat.replicate r) $ randMoves v s g
    SourceFile f -> do { s <- f; return (readListLn s) }
  let curves1 = map getCurve tests
      curves = case optExtraTwists opts of
	 Nothing -> curves1
	 Just n -> addExtraTwists n g curves1
  res <- sequence (map (doTest opts) curves)
  printSummary res
  exitWith ExitSuccess
\end{code}

Copyright 2002, 2004, 2005, 2006 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
