\begin{code}
module RandomGenus2 where
import Genus2
import DT
import Random
import PIL((//))
\end{code}

The goal of this module is to pick a random integer measured
lamination on a genus 2 surface, with respect to a pair of pants
decomposition with respect to the barbell graph

         __	    __
        /  \	   /  \
       /    \  c  /    \
     a |    +-----+    | b
       \    /	  \    /
        \__/ 	   \__/

subject to the constraint that the total number of crossings of a and
b be less than or equal R (a given parameter) and that there aren't
too many crossings of c.  We need to pick the number of crossings m(x)
of each pants curve and the twist t(x) along that curve.
(Specifically, we require that m(c) <= 2*min(m(a),m(b)).  This amounts
to requiring that the given pants decomposition is close to the
simplest one that we can reach without changing the Heegard
splitting.)

The code is slightly complicated because we pick on a uniform
distribution among all possibilities, and not all (a,b,c) triples have
equal numbers of possibilities.

In general, we may wish to pick integers according to some weight
function.

\begin{code}
type Weight = Integer

data WeightFunc = WeightFunc {minWF, maxWF :: Integer,
			      cumWeight :: Integer -> Weight}

totWeight :: WeightFunc -> Weight
totWeight wf = cumWeight wf (maxWF wf)

pickCum :: WeightFunc -> Weight -> (Weight, Integer)
pickCum wf w = aux (minWF wf - 1) (maxWF wf)
  where cw = cumWeight wf
	aux mn mx | mx - mn <= 1 = (w - cw mn, mx)
		  | otherwise    =
		      let mid = (mn + mx) `div` 2
		      in if w < cw mid then aux mn mid else aux mid mx
\end{code}

Probability distribution for c given a, b.  For given c, the number of
possibilities is is a*b*c for c even, 0 otherwise, since once we've
picked a, b, and c, all that's left is to pick the twist modulo a full
twist.

\begin{code}
cWF :: Integer -> Integer -> WeightFunc
cWF a b | b < a = cWF b a
cWF a b = WeightFunc {minWF = 0, maxWF = 2*a,
		      cumWeight = \c -> let d = c `div` 2 in
		                        a*b*d*(d+1)}
\end{code}

Probability distribution for a given total weight. For given a, number
of c possibilities is a*a*(a+1)*(t-a).  The expression in the code
below is courtesy of Maxima, and comes from summing the total number
of c possibilities.

\begin{code}
aWF :: Integer -> WeightFunc
aWF t = let cw x = x*(x+1)*(x+2)*(-12*x*x+15*t*x-9*x+5*t+1) // 60
	in WeightFunc {minWF = 0, maxWF = t `div` 2, cumWeight = cw}
\end{code}

Finally, the total number of possibilities for a given value of t=a+b
is (t-1)*t*(t+1)*(t+2)*(3t-1)/60, and the cumulative weights are given
by summing this.  Again, the expression is courtesy of Maxima.

\begin{code}
tWF :: Integer -> WeightFunc
tWF t = let tw1 x = x*(x+1)*(x+2)*(x+3)*(2*x+1)*(3*x+2)//60
	    tw2 x = x*x*(x+1)*(x+2)*(6*x*x+7*x+7)//60
        in WeightFunc {minWF = 0, maxWF = t,
		       cumWeight = \x -> if even x then tw2 (x`div`2) else tw1 (x`div`2)}
\end{code}

\begin{code}
pickGenus2 :: Integer -> Weight -> Genus2
pickGenus2 t1 w =
  let (w1, t) = pickCum (tWF t1) w
      (w2, a) = pickCum (aWF t) w1
      b = t - a
      (w3, c) = pickCum (cWF a b) w2
      (w4, alpha) = w3 `divMod` a
      (gamma, beta) = w4 `divMod` b
  in Barbell (DTAnnulus {m = a, t = 2*alpha})
	     (DTAnnulus {m = c, t = 2*gamma})
	     (DTAnnulus {m = b, t = 2*beta})

randCurves :: RandomGen g => Integer -> g -> [Genus2]
randCurves t g = let tw = totWeight (tWF t)
		     xs = randomRs (0,tw-1) g
		 in map (pickGenus2 t) xs
\end{code}

Copyright 2002, 2004 Dylan Thurston.

This file is part of the genus2fiber program for computing what
percentage of tunnel-number one 3-manifolds fiber.  It can be
redistributed and/or modified under the terms of the GNU General
Public License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.
